@@ -18,6 +18,7 @@
 ;;;;
 
 (define-module (fibers scheduler)
+  #:autoload   (srfi srfi-1) (remove fold)
   #:use-module (srfi srfi-9)
   #:use-module (srfi srfi-9 gnu)
   #:use-module (fibers events-impl)
@@ -44,6 +45,7 @@
             schedule-task
             schedule-task-when-fd-readable
             schedule-task-when-fd-writable
+            unschedule-task-for-fd
             schedule-task-at-time
 
             rewinding-for-scheduling?
@@ -368,7 +370,36 @@ value.  Return zero values."
        (unless (= (logand events active-events) events)
          (let ((active-events (logior events active-events)))
            (set-car! fd-waiters active-events)
-           (events-impl-add! (scheduler-events-impl sched) fd active-events)))))))
+           (events-impl-add! (scheduler-events-impl sched) fd
+                             active-events)))))))
+
+(define (unschedule-task-for-fd sched fd task)
+  "Remove @var{task}, a thunk scheduled to run on @var{fd} activity, from the
+list of \"file descriptor waiters\" of @var{sched}."
+  (let ((fd-waiters (hashv-ref (scheduler-fd-waiters sched) fd)))
+    (match fd-waiters
+      ((active-events . waiters)
+       (let* ((waiters (remove (match-lambda
+                                 ((_ . t)
+                                  (eq? t task))
+                                 (_ #f))
+                               waiters))
+              (remaining-events (fold (lambda (event+waiter result)
+                                        (match event+waiter
+                                          ((event . _)
+                                           (logior event result))))
+                                      0
+                                      waiters)))
+         (when (null? waiters)
+           ;; Since the FD finalizer cannot easily be removed (we would need
+           ;; to hold a reference to the finalizer as it was given to
+           ;; 'add-fdes-finalizer!'), keep the hash table entry, with zero
+           ;; waiters, but remove polling for FD.
+           (events-impl-remove! (scheduler-events-impl sched) fd))
+
+         (set-car! fd-waiters remaining-events)
+         (set-cdr! fd-waiters waiters)))
+      (_ #f))))
 
 (define (schedule-task-when-fd-readable sched fd task)
   "Arrange to schedule @var{task} on @var{sched} when the file
